/** @file runtime/event.h
 * Common definitions for code generated for transitions
 */

/** flag: has the analysis been interrupted? */
extern volatile char* intr;
/** flag: has a fatal error occurred? */
extern char* fatal;
/** flag: generate arcs? */
extern char arcs;
/** weak fairness sets */
extern unsigned wfair[];
/** strong fairness sets */
extern unsigned sfair[];

/** encode the marking and add an event from the current state to it
 * @param tr	the index number of the transition that fired
 * @param hide	flag: is the transition to the state hidden?
 * @param ctx	the call-back context
 */
enum Error
encode (unsigned tr, int hide, void* ctx);

/** append data to the event encoding buffer
 * @param data	data to append
 * @param bits	number of bits (data < 1 << bits)
 */
extern void
enc (card_t data, unsigned bits);
/** extract data from the event encoding buffer
 * @param bits	number of bits to extract
 * @return	the extracted data (< 1 << bits)
 */
extern card_t
dec (unsigned bits);
/** clear the event encoding buffer */
extern void
event_clear (void);
/** clean up the event encoding buffer */
extern void
event_cleanup (void);
/** set up and inflate the event decoding buffer
 * @param buf	the decoding buffer
 * @param bytes	length of the decoding buffer
 */
extern void
inflate (void* buf, size_t bytes);
/** deflate the event encoding buffer
 * @param size	(output) number of bytes in the encoded buffer
 * @return	the start address of the encoding buffer
 */
extern void*
deflate (size_t *size);
/** decode an event and compute the fairness sets
 * @param sf	flag: compute strong fairness constraints
 * @return	number of the transition
 */
extern enum Error
event_decode (unsigned sf);

/** See if an optional variable has been assigned a value
 * @param i	the bit vector holding the "assigned" statuses
 * @param v	index number of the variable
 */
#define ASSIGNED(i,v) (i.y[v / sizeof *i.y] & (1 << (v % sizeof *i.y)))
/** Flag that an optional variable has been assigned a value
 * @param i	the bit vector holding the "assigned" statuses
 * @param v	index number of the variable
 */
#define ASSIGN(i,v) (i.y[v / sizeof *i.y] |= (1 << (v % sizeof *i.y)))
/** Flag that an optional variable has no assigned value
 * @param i	the bit vector holding the "assigned" statuses
 * @param v	index number of the variable
 */
#define DEASSIGN(i,v) (i.y[v / sizeof *i.y] &= ~(1 << (v % sizeof *i.y)))
