// Function definition -*- c++ -*-

#ifndef FUNCTION_H_
# define FUNCTION_H_
# ifdef __GNUC__
#  pragma interface
# endif // __GNUC__

# include <assert.h>

/** @file Function.h
 * Function (typed macro) definition
 */

/** Function definition */
class Function
{
public:
  /** Constructor
   * @param name	Name of the function
   * @param arity	Arity (number of function parameters)
   * @param params	Parameter list of the function
   * @param expr	Function definition
   */
  Function (char* name, unsigned arity, class VariableDefinition** params,
	    class Expression* expr);
private:
  /** Copy constructor */
  Function (const class Function& old);
  /** Assignment operator */
  class Function& operator= (const class Function& old);
public:
  /** Destructor */
  ~Function ();

  /** Determine the name of the function */
  const char* getName () const { return myName; }

  /** Determine the arity of the function */
  unsigned getArity () const { return myArity; }

  /** Get a formal parameter */
  const class VariableDefinition& operator[] (unsigned i) const {
    assert (i < myArity); return *myParams[i];
  }

  /** Get the expression */
  const class Expression& getExpr () const { return *myExpr; }

  /** Substitute the expression */
  void substitute (class Substitution& substitution);

  /** Expand the function
   * @param params	The expression list
   * @return		The expanded function, or NULL in case of error
   */
  class Expression* expand (class ExpressionList* params);

private:
  /** Name of the function */
  char* myName;
  /** Arity of the function */
  unsigned myArity;
  /** Function parameters */
  class VariableDefinition** myParams;
  /** Function definition */
  class Expression* myExpr;
};

#endif // FUNCTION_H_
