// Arc class -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "Arc.h"
#include "Marking.h"
#include "Place.h"

/** @file Arc.C
 * Labelled arc in a net
 */

Arc::Arc (bool output,
	  class Marking& expr,
	  const class Place& place) :
  myIsOutput (output), myExpr (&expr),
  myPlace (place), myIndex (place.getIndex ())
{
  expr.setPlace (&place);
}

Arc::~Arc ()
{
  myExpr->destroy ();
}

void
Arc::substitute (class Substitution& substitution)
{
  class Expression* expr = myExpr->substitute (substitution);
  assert (expr && expr->getKind () == Expression::eMarking);
  myExpr->destroy ();
  myExpr = static_cast<class Marking*>(expr);
}

void
Arc::append (class Marking& expr)
{
  expr.setPlace (&myPlace);
  myExpr->append (expr);
}
